﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

using System.IO;

namespace FolderBrowserDialog {
    /// <summary>
    /// Interaction logic for WSelectDirectory.xaml
    /// </summary>
    public partial class WSelectDirectory : Window {

        private bool retcode = false;

        public bool Retcode {
            get { return retcode; }
        }

        public String SelectDirectory {
            get { return tDirectory.Text.Trim(); }
        }

        private String initialDirectory = "";
        public String InitialDirectory {
            set {
                if (Directory.Exists(value)) {
                    initialDirectory = value;
                }
            }
        }

        public WSelectDirectory(Window parent, String startPath) {
            InitializeComponent();
            this.Owner = parent;  // Important, because CenterOwner
            this.WindowStartupLocation = WindowStartupLocation.CenterOwner; // nur in XAML            
            InitialDirectory = startPath;
        }


        private void Window_Loaded(object sender, RoutedEventArgs e) {
            //Image image1 = imageDisk;

            TreeViewItem root = new TreeViewItem();
            root.Header = "Computer";
            //root.FontWeight = FontWeights.Bold;
            tree.Items.Add(root);

            TreeViewItem user = new TreeViewItem();
            user.Header = "User";
            //user.FontWeight = FontWeights.Bold;
            root.Items.Add(user);
            insertUserDirectories(user);
            user.IsExpanded = true;

            TreeViewItem drives = new TreeViewItem();
            drives.Header = "Laufwerke";
            //drives.FontWeight = FontWeights.Bold;
            root.Items.Add(drives);
            insertDrives(drives);
            drives.IsExpanded = true;
            root.IsExpanded = true;

            if (Directory.Exists(initialDirectory)) {
                if (initialDirectory.StartsWith("C:\\Users\\")) {
                    searchDirectory(user, initialDirectory);
                }
                else {
                    // Search path in the drives
                    searchDirectory(drives, initialDirectory);
                }
            }

        }

        private void searchDirectory(TreeViewItem root, String dir) {
            root.IsExpanded = true;
            // C:\Users\Kraft-300\Desktop\igm
            bool aktive = true;
            foreach (TreeViewItem node in root.Items) {
                if (dir.StartsWith((String)node.Tag)) {
                    searchDirectory(node, dir);   // "E:\\mlu\\doks\\html\\brit\\datum_files"  vs. "E:\\mlu\\doks\\html\\brit\\datum_files\\"
                    aktive = false;
                }
            }
            if (aktive) {
                root.IsSelected = true;
                root.BringIntoView();
                tree.Focus();
            }
        }

        private void insertUserDirectories(TreeViewItem mainNode) {
            TreeViewItem node1 = new TreeViewItem();
            node1.Tag = Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory);
            node1.Header = "Desktop";
            mainNode.Items.Add(node1);

            node1 = new TreeViewItem();
            node1.Tag = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
            node1.Header = "Dokumente";
            mainNode.Items.Add(node1);

            node1 = new TreeViewItem();
            node1.Tag = Environment.GetFolderPath(Environment.SpecialFolder.MyMusic);
            node1.Header = "Musik";
            mainNode.Items.Add(node1);

            node1 = new TreeViewItem();
            node1.Tag = Environment.GetFolderPath(Environment.SpecialFolder.MyVideos);
            node1.Header = "Videos/Filme";
            mainNode.Items.Add(node1);

            node1 = new TreeViewItem();
            node1.Tag = Environment.GetFolderPath(Environment.SpecialFolder.CommonPictures);
            node1.Header = "Bilder";
            mainNode.Items.Add(node1);

        }

        public String getProgPath() {
            string[] args = System.Environment.GetCommandLineArgs();
            String exepath = args[0];
            int i = exepath.LastIndexOf("\\");
            String path = exepath.Substring(0, i + 1);
            if (!path.EndsWith("\\")) path += "\\";
            return path;
        }


        private void insertDrives(TreeViewItem mainNodeDrives) {
            // TreeViewItem node1;
            mainNodeDrives.Items.Clear();
            // String[] drives = System.Environment.GetLogicalDrives();  // get only the Strings
            DriveInfo[] drives = DriveInfo.GetDrives();
            int lfdnr = 0;
            foreach (DriveInfo drv in drives) {
                lfdnr++;
                TreeViewItem node1 = new TreeViewItem();
                node1.IsTabStop = false;
                node1.Tag = drv.RootDirectory.FullName;
                StackPanel panel = new StackPanel();
                panel.Orientation = Orientation.Horizontal;

                TextBlock text4 = new TextBlock();
                Image bild1 = new Image();
                String typ = "";
                switch (drv.DriveType) {
                    case DriveType.CDRom: typ = "CD/DVD:";
                        bild1.Source = imageCD.Source;
                        break;
                    case DriveType.Fixed: typ = "Platte:";
                        bild1.Source = imageDisk.Source;
                        break;
                    case DriveType.Network: typ = "Netz:";
                        bild1.Source = imageNetwork.Source;
                        break;

                    case DriveType.Ram: typ = "Ram:"; break;
                    case DriveType.Removable: typ = "USB:";
                        bild1.Source = imageFloppy.Source;
                        break;
                    case DriveType.Unknown: typ = "?:"; break;
                }
                if (drv.DriveType != DriveType.CDRom && drv.DriveType != DriveType.Removable)
                    if (drv.VolumeLabel.Equals("")) {
                        typ = typ + " (" + drv.RootDirectory + ")";
                    }
                    else {
                        typ = drv.VolumeLabel + " (" + drv.RootDirectory + ")";
                    }
                else {
                    if (drv.DriveType == DriveType.CDRom)
                        typ = typ + " (" + drv.RootDirectory + ")";
                    else
                        typ = typ + " (" + drv.RootDirectory + ")";
                }
                //text4.Text = typ + "  Alt+" + lfdnr.ToString();
                text4.Text = drv.RootDirectory.ToString();

                // check subDirs
                if (drv.IsReady) {
                    String[] dirs = Directory.GetDirectories(drv.RootDirectory.FullName);
                    int n = 0;
                    foreach (String item in dirs) {
                        DirectoryInfo dir = new DirectoryInfo(item);
                        FileAttributes attribs = dir.Attributes;
                        if ((attribs & FileAttributes.Hidden) != FileAttributes.Hidden && (attribs & FileAttributes.System) != FileAttributes.System) {
                            n++;
                        }
                    }
                    if (n > 0) {
                        TreeViewItem subNode = new TreeViewItem();
                        subNode.Header = "?";
                        node1.Items.Add(subNode);
                    }
                }

                //bild1.Width = 19;
                //bild1.Height = 12;
                panel.Children.Add(bild1);
                panel.Children.Add(text4);
                node1.Header = panel;
                mainNodeDrives.Items.Add(node1);
            }  // foreach
        }

        private void tree_SelectedItemChanged(object sender, RoutedPropertyChangedEventArgs<object> e) {
            tree_click();
            e.Handled = true; // abgearbeitet
        }
        private void TreeViewItem_Expanded(object sender, RoutedEventArgs e) {
            TreeViewItem node1 = e.OriginalSource as TreeViewItem;
            if (node1 != null) {
                insertSubDirsIntoTree(node1, false);
            }
        }


        private void tree_click() {
            TreeViewItem node1 = (TreeViewItem)tree.SelectedItem;
            if (node1 == null) return;
            String path = (String)node1.Tag;
            if (path == null) {
                //MessageBox.Show("Das Laufwerk ist null", "Wechseln in ein Laufwerk", MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }
            if (Directory.Exists(path)) {
                if (path.Length <= 3) {
                    DriveInfo drv = new DriveInfo(path);
                    if (!drv.IsReady) {
                        MessageBox.Show("Das Laufwerk ist nicht bereit\r\n" + path, "Wechseln in ein Laufwerk", MessageBoxButton.OK, MessageBoxImage.Error);
                        return;
                    }
                }
                //_focusfileTable.ActualPath = path;
            }
            insertSubDirsIntoTree(node1, false);
        }


        public void insertSubDirsIntoTree(TreeViewItem node1, bool clearSubs) {
            TreeViewItem node2, node3;
            if (node1 != null) {
                String path = (String)node1.Tag;
                tDirectory.Text = path;
                if (node1.Tag != null && !Directory.Exists(path)) {
                    // Basis.ErrorMsg("Das Verzeichnis ist nicht vorhanden\r\n" + path, "Wechseln in ein Verzeichnis");
                    return;
                }
                bool search = false;
                if (clearSubs && node1.HasItems) {
                    node1.Items.Clear();
                    search = true;
                }
                else {
                    // check, if the node hat Items, BUT node with subdirs has at first only ONE node with "?"
                    // if the node hat Items, don't search again
                    search = !node1.HasItems;
                    if (!search) {
                        // test if item ist "?", ween search
                        node2 = (TreeViewItem)node1.Items[0];
                        if (node2.Header.Equals("?")) {
                            search = true;
                            node1.Items.Clear();
                        }
                    }
                    else {
                    }
                }

                if (search) {
                    try {
                        // search for subdirs
                        String[] dirs = Directory.GetDirectories(path);
                        foreach (String item in dirs) {
                            DirectoryInfo dir = new DirectoryInfo(item);
                            FileAttributes attribs = dir.Attributes;
                            if ((attribs & FileAttributes.Hidden) != FileAttributes.Hidden && (attribs & FileAttributes.System) != FileAttributes.System) {
                                node2 = new TreeViewItem();
                                node2.IsTabStop = false;



                                StackPanel panel = new StackPanel();
                                panel.Orientation = Orientation.Horizontal;

                                TextBlock text4 = new TextBlock();
                                text4.Text = dir.Name;
                                Image bild1 = new Image();
                                BitmapImage bi1 = new BitmapImage();
                                bi1.BeginInit();
                                bi1.UriSource = new Uri("directory.png", UriKind.RelativeOrAbsolute);
                                bi1.EndInit();
                                bild1.Source = bi1;
                                double w = bi1.Width;
                                double h = bi1.Height;
                                panel.Children.Add(bild1);
                                panel.Children.Add(text4);
                                node2.Header = panel;
                                node2.Tag = dir.FullName;
                                node1.Items.Add(node2);
                                try {
                                    String[] dirs2 = Directory.GetDirectories(dir.FullName);
                                    if (dirs2.Length > 0) {
                                        node3 = new TreeViewItem();
                                        node3.IsTabStop = false;
                                        node3.Header = "?";
                                        node3.Tag = null;
                                        node2.Items.Add(node3);
                                    }
                                }
                                catch { }
                            }
                        }
                        node1.IsExpanded = true;
                    }
                    catch (Exception e) {
                        MessageBox.Show(e.ToString(), "Fehler", MessageBoxButton.OK, MessageBoxImage.Error);
                    }
                }
                // showPath(path);  // TreeViewList, falsch, da diese Routine auch von expanded aufgerufen wird
            }
            else {
                // Title = "node1 == null";
            }
        }  // insertSubDirintoTrees




        private void bnNewDirectory_Click(object sender, RoutedEventArgs e) {
            if (tDirectory.Text.Length > 0) {
                WInsertDirectory dialog = new WInsertDirectory(this);
                dialog.ShowDialog();
                if (dialog.Retcode) {
                    String newPath;
                    if (!tDirectory.Text.EndsWith("\\"))
                        newPath = tDirectory.Text + "\\" + dialog.NewDirectory;
                    else
                        newPath = tDirectory.Text + dialog.NewDirectory;
                    Directory.CreateDirectory(newPath);
                    TreeViewItem node1 = (TreeViewItem)tree.SelectedItem;
                    if (node1 != null) {
                        insertSubDirsIntoTree(node1, true);
                    }
                }
            }

        }

        private void bnOk_Click(object sender, RoutedEventArgs e) {
            if (Directory.Exists(tDirectory.Text)) {
                retcode = true;
                Close();
            }
            else {
                MessageBox.Show("Bitte wählen Sie ein Verzeichnis aus", "Auswahl eines Verzeichnisses", MessageBoxButton.OK, MessageBoxImage.Information);

            }
        }

        private void bnEsc_Click(object sender, RoutedEventArgs e) {
            Close();
        }
    }
}
